#!/usr/local/bin/perl
use Benchmark;

# Each run is killed if a timeout is exceeded, next run is started immediately.
# If a run completes before timeout, the next run is started immediately.
# Author: Himanshu Jain (hjain@cs.cmu.edu)
# Carnegie Mellon University, Computer Science Department

print "Usage: timedRunSolver.pl log_file solver_name timeout benchmark_file additional_options\n";

$user=`whoami`;

print "User $user\n";

if ($ARGV[0] eq "")
{
    die "Must provide a first argument a log file. This is where the results will be written.\n";
}


if ($ARGV[1] eq "")
{
    die "Must provide solver name. Either give absolute path to the solver or make sure the solver is included in PATH \n";
}


if ($ARGV[2] eq "")
{
    die "Must provide timeout in seconds. This is timeout will be applied to each benchmark.\n";
}


if ($ARGV[3] eq "")
{
    die "Must provide a filename containing benchmarks. Each line in the file should list the name of the benchmark. The name of the benchmark can be absolute path or just call the timedRunSolver.pl from the directory where all files are present.\n";
}

print "additional options are $ARGV[4]\n";


$filename = $ARGV[3];
open (READ_DATA, $filename);


$solver = $ARGV[1];

print "using solver $solver \n";

$time = 1800; #in seconds, timeout

if (!($ARGV[2] eq ""))
{
    $time = $ARGV[2];
}

print "timeout = $time seconds \n";

$extension = ".".$solver."_".$time;

$writefile= $ARGV[0].$extension;
open (WRITEFILE, ">".$writefile);


print WRITEFILE "Log files in $ARGV[0]\n";
print WRITEFILE "Solver name: $solver \n";
print WRITEFILE "Timeout $time \n";
print WRITEFILE "Benchmark list in $filename \n";
print WRITEFILE "Additional options $ARGV[4] \n";

print WRITEFILE "\n------------------\n";

$line = <READ_DATA>;
$command = " ";

while ($line ne "") {
    chop ($line);

    print "Handling $line\n";

    $bzipused =false;

    $cnf_file_name = $line;

    $addition="";

    if (($line =~ /$\.iscas/) || ($line =~ /\.bench/))
    {
	$addition="--iscas";
    }
    elsif (($line =~ /$\.aig/) || ($line =~ /\.AIG/))
    {
	$addition="--aig";
    }
    
    print "discovered extension $addition\n";

    $command1 = "$solver $cnf_file_name $ARGV[4] $addition";

    print "$command \n";

    $procid = fork();
    if ($procid == 0) { #The kid

	my $start_verif = new Benchmark;
	@out = `$command1`;
	my $end_verif = new Benchmark;

	$solving_time = timediff($end_verif, $start_verif);

	print WRITEFILE "PERLBENCH SUCCESS $cnf_file_name ",timestr($solving_time),"\n";
	print WRITEFILE "$command1...\n";
	print WRITEFILE @out;
	print WRITEFILE "\n------------------\n";
	exit;
    }
    else { #I am a parent.
	$SIG{ALRM} = sub { 
	    print "...$command1 exceeded alotted time\n";
	    
	    print WRITEFILE "PERLBENCH KILLED $cnf_file_name ",$timeout,"\n";
	    print WRITEFILE "$command1 killed...\n";
	    print WRITEFILE "\n------------------\n";

	    kill (9, $procid);
	    print ("procid is $procid");
	};
	alarm ($time);
	wait;
    }
    
    @list = `ps -fu $user`; 
    foreach $nline (@list) 
    { 
	if($nline =~ /$solver/ && !($nline =~ /timedRunSolver/) )
	{ 
	    print "Killing $nline";
	    chop($nline);
	    @b = split(/ +/, $nline);
	    print "killing $b[1]\n";
	    kill 9, $b[1];
	}
    } 
    

    $line = <READ_DATA>;
}

close(READ_DATA);
close(WRITEFILE);

